@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2006  Niklas Knutsson (nq@altern.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 							gtk_tree_view_expand_to_path(GTK_TREE_VIEW(view), path); \
 							gtk_tree_path_free(path);
 #define EXPAND_ITER(model, view, iter)			GtkTreePath *path = gtk_tree_model_get_path(model, &iter); \
-							gtk_tree_view_expand_row(GTK_TREE_VIEW(view), path, false); \
+							gtk_tree_view_expand_row(GTK_TREE_VIEW(view), path, FALSE); \
 							gtk_tree_path_free(path);							
 
 #include "main.h"
@@ -30,6 +30,7 @@ struct mode_struct {
 	AssumptionSign as;
 	int precision;
 	string name;
+	bool rpn_mode;
 };
 
 enum {
@@ -89,8 +90,15 @@ void on_tDataProperties_selection_changed(GtkTreeSelection *treeselection, gpoin
 void on_tUnitSelector_selection_changed(GtkTreeSelection *treeselection, gpointer user_data);
 void on_tUnitSelectorCategories_selection_changed(GtkTreeSelection *treeselection, gpointer user_data);
 
-void execute_expression(bool force = true);
-void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, string transformation = "");
+void execute_expression(bool force = true, bool do_mathoperation = false, MathOperation op = OPERATION_ADD, MathFunction *f = NULL, bool do_stack = false, size_t stack_index = 0);
+void setResult(Prefix *prefix = NULL, bool update_history = true, bool update_parse = false, bool force = false, string transformation = "", size_t stack_index = 0, bool register_moved = false);
+
+void set_rpn_mode(bool b);
+void calculateRPN(int op);
+void calculateRPN(MathFunction *f);
+void RPNRegisterAdded(string text, gint index = 0);
+void RPNRegisterRemoved(gint index);
+void RPNRegisterChanged(string text, gint index);
 
 void result_display_updated();
 void result_format_updated();
@@ -144,6 +152,7 @@ void set_variables_enabled(GtkMenuItem *w, gpointer user_data);
 void set_donot_calcvars(GtkMenuItem *w, gpointer user_data);
 void set_unknownvariables_enabled(GtkMenuItem *w, gpointer user_data);
 void set_units_enabled(GtkMenuItem *w, gpointer user_data);
+void apply_function(GtkMenuItem *w, gpointer user_data);
 void insert_function(GtkMenuItem *w, gpointer user_data);
 void insert_variable(GtkMenuItem *w, gpointer user_data);
 void insert_prefix(GtkMenuItem *w, gpointer user_data);
@@ -162,6 +171,7 @@ void edit_argument(Argument *arg = NULL);
 void edit_unknown(const char *category = "", Variable *v = NULL, GtkWidget *win = NULL);
 void edit_variable(const char *category = "", Variable *v = NULL, MathStructure *mstruct_ = NULL, GtkWidget *win = NULL);
 void edit_matrix(const char *category = "", Variable *v = NULL, MathStructure *mstruct_ = NULL, GtkWidget *win = NULL, gboolean create_vector = FALSE);
+void insert_matrix(const MathStructure *initial_value = NULL, GtkWidget *win = NULL, gboolean create_vector = FALSE, bool is_text_struct = false, bool is_result = false);
 void import_csv_file(GtkWidget *win = NULL);
 void export_csv_file(KnownVariable *v = NULL, GtkWidget *win = NULL);
 
@@ -187,6 +197,7 @@ void on_combobox_base_changed(GtkComboBox *w, gpointer user_data);
 void on_combobox_numerical_display_changed(GtkComboBox *w, gpointer user_data);
 void on_expander_keypad_expanded(GObject *o, GParamSpec *param_spec, gpointer user_data);
 void on_expander_history_expanded(GObject *o, GParamSpec *param_spec, gpointer user_data);
+void on_expander_stack_expanded(GObject *o, GParamSpec *param_spec, gpointer user_data);
 void on_menu_item_meta_mode_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_meta_mode_save_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_meta_mode_delete_activate(GtkMenuItem *w, gpointer user_data);
@@ -196,6 +207,7 @@ void on_colorbutton_status_warning_color_color_set(GtkColorButton *w, gpointer u
 void on_preferences_entry_wget_args_changed(GtkEditable *w, gpointer user_data);
 void on_preferences_checkbutton_lower_case_numbers_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_lower_case_e_toggled(GtkToggleButton *w, gpointer user_data);
+void on_preferences_checkbutton_alternative_base_prefixes_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_spell_out_logical_operators_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_unicode_signs_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_display_expression_status_toggled(GtkToggleButton *w, gpointer user_data);
@@ -203,6 +215,8 @@ void on_preferences_checkbutton_enable_completion_toggled(GtkToggleButton *w, gp
 void on_preferences_checkbutton_fetch_exchange_rates_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_save_defs_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_save_mode_toggled(GtkToggleButton *w, gpointer user_data);
+void on_preferences_checkbutton_rpn_keypad_only_toggled(GtkToggleButton *w, gpointer);
+void on_preferences_checkbutton_dot_as_separator_toggled(GtkToggleButton *w, gpointer);
 void on_preferences_checkbutton_load_defs_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_custom_result_font_toggled(GtkToggleButton *w, gpointer user_data);
 void on_preferences_checkbutton_custom_expression_font_toggled(GtkToggleButton *w, gpointer user_data);
@@ -280,6 +294,8 @@ void on_menu_item_export_csv_file_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_convert_to_unit_expression_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_convert_to_base_units_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_convert_to_best_unit_activate(GtkMenuItem *w, gpointer user_data);
+void on_menu_item_insert_matrix_activate(GtkMenuItem *w, gpointer user_data);
+void on_menu_item_insert_vector_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_enable_variables_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_enable_functions_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_enable_units_activate(GtkMenuItem *w, gpointer user_data);
@@ -295,6 +311,7 @@ void on_menu_item_new_function_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_new_dataset_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_new_unit_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_rpn_mode_activate(GtkMenuItem *w, gpointer user_data);
+void on_menu_item_rpn_syntax_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_limit_implicit_multiplication_activate(GtkMenuItem *w, gpointer user_data);
 void fetch_exchange_rates(int timeout);
 void on_menu_item_fetch_exchange_rates_activate(GtkMenuItem *w, gpointer user_data);
@@ -305,6 +322,8 @@ void on_menu_item_degrees_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_radians_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_gradians_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_no_default_angle_unit_activate(GtkMenuItem *w, gpointer user_data);
+void set_output_base_from_dialog(int base);
+void output_base_updated_from_menu();
 void on_menu_item_binary_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_octal_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_decimal_activate(GtkMenuItem *w, gpointer user_data);
@@ -314,14 +333,23 @@ void on_menu_item_custom_base_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_roman_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_sexagesimal_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_time_format_activate(GtkMenuItem *w, gpointer user_data);
-void on_menu_item_expression_base_activate(GtkMenuItem *w, gpointer user_data);
-void on_number_base_expression_radiobutton_binary_toggled(GtkToggleButton *w, gpointer user_data);
-void on_number_base_expression_radiobutton_octal_toggled(GtkToggleButton *w, gpointer user_data);
-void on_number_base_expression_radiobutton_decimal_toggled(GtkToggleButton *w, gpointer user_data);
-void on_number_base_expression_radiobutton_hexadecimal_toggled(GtkToggleButton *w, gpointer user_data);
-void on_number_base_expression_radiobutton_custom_base_toggled(GtkToggleButton *w, gpointer user_data);
-void on_number_base_expression_spinbutton_base_value_changed(GtkSpinButton *w, gpointer user_data);
-void on_number_base_expression_radiobutton_roman_toggled(GtkToggleButton *w, gpointer user_data);
+void on_menu_item_set_base_activate(GtkMenuItem *w, gpointer user_data);
+void on_set_base_radiobutton_output_binary_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_output_octal_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_output_decimal_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_output_hexadecimal_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_output_other_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_output_sexagesimal_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_output_time_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_output_roman_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_spinbutton_output_other_value_changed(GtkSpinButton *w, gpointer user_data);
+void on_set_base_radiobutton_input_binary_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_input_octal_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_input_decimal_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_input_hexadecimal_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_input_other_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_radiobutton_input_roman_toggled(GtkToggleButton *w, gpointer user_data);
+void on_set_base_spinbutton_input_other_value_changed(GtkSpinButton *w, gpointer user_data);
 void on_menu_item_convert_number_bases_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_periodic_table_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_assumptions_integer_activate(GtkMenuItem *w, gpointer user_data);
@@ -358,6 +386,8 @@ void on_popup_menu_item_custom_base_activate(GtkMenuItem *w, gpointer user_data)
 void on_popup_menu_item_abbreviate_names_activate(GtkMenuItem *w, gpointer user_data);
 void on_popup_menu_item_all_prefixes_activate(GtkMenuItem *w, gpointer user_data);
 void on_popup_menu_item_denominator_prefixes_activate(GtkMenuItem *w, gpointer user_data);
+void on_popup_menu_item_view_matrix_activate(GtkMenuItem *w, gpointer user_data);
+void on_popup_menu_item_view_vector_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_display_normal_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_display_scientific_activate(GtkMenuItem *w, gpointer user_data);
 void on_menu_item_display_purely_scientific_activate(GtkMenuItem *w, gpointer user_data);
@@ -392,6 +422,14 @@ void on_menu_item_algebraic_mode_factorize_activate(GtkMenuItem *w, gpointer use
 void on_menu_item_algebraic_mode_none_activate(GtkMenuItem *w, gpointer user_data);
 gboolean on_main_window_focus_in_event(GtkWidget *w, GdkEventFocus *e, gpointer user_data);
 
+void on_button_registerup_clicked(GtkButton *button, gpointer user_data);
+void on_button_registerdown_clicked(GtkButton *button, gpointer user_data);
+void on_button_editregister_clicked(GtkButton *button, gpointer user_data);
+void on_button_deleteregister_clicked(GtkButton *button, gpointer user_data);
+void on_button_clearstack_clicked(GtkButton *button, gpointer user_data);
+void on_stackview_selection_changed(GtkTreeSelection *treeselection, gpointer user_data);
+void on_stackview_item_edited(GtkCellRendererText *renderer, gchar *path, gchar *new_text, gpointer user_data);
+
 void on_unit_edit_entry_name_changed(GtkEditable *editable, gpointer user_data);
 void on_unit_edit_optionmenu_class_changed(GtkOptionMenu *om, gpointer user_data);
 void on_units_button_new_clicked(GtkButton *button, gpointer user_data);
@@ -408,6 +446,7 @@ void on_variables_button_close_clicked(GtkButton *button, gpointer user_data);
 void on_functions_button_new_clicked(GtkButton *button, gpointer user_data);
 void on_functions_button_edit_clicked(GtkButton *button, gpointer user_data);
 void on_functions_button_insert_clicked(GtkButton *button, gpointer user_data);
+void on_functions_button_apply_clicked(GtkButton *button, gpointer user_data);
 void on_functions_button_delete_clicked(GtkButton *button, gpointer user_data);
 void on_functions_button_close_clicked(GtkButton *button, gpointer user_data);
 void on_datasets_button_close_clicked(GtkButton *button, gpointer user_data);
@@ -458,12 +497,26 @@ gboolean on_expression_key_press_event(GtkWidget *w, GdkEventKey *event, gpointe
 
 gboolean on_resultview_expose_event(GtkWidget *w, GdkEventExpose *event, gpointer user_data);
 
+gboolean on_tMatrixEdit_key_press_event(GtkWidget *w, GdkEventKey *event, gpointer user_data);
+gboolean on_tMatrixEdit_button_press_event(GtkWidget *w, GdkEventButton *event, gpointer user_data);
+gboolean on_tMatrixEdit_cursor_changed(GtkTreeView *w, gpointer user_data);
+
 void on_matrix_edit_spinbutton_columns_value_changed(GtkSpinButton *w, gpointer user_data);
 void on_matrix_edit_spinbutton_rows_value_changed(GtkSpinButton *w, gpointer user_data);
 
 void on_matrix_edit_radiobutton_matrix_toggled(GtkToggleButton *w, gpointer user_data);
 void on_matrix_edit_radiobutton_vector_toggled(GtkToggleButton *w, gpointer user_data);
 
+gboolean on_tMatrix_key_press_event(GtkWidget *w, GdkEventKey *event, gpointer user_data);
+gboolean on_tMatrix_button_press_event(GtkWidget *w, GdkEventButton *event, gpointer user_data);
+gboolean on_tMatrix_cursor_changed(GtkTreeView *w, gpointer user_data);
+
+void on_matrix_spinbutton_columns_value_changed(GtkSpinButton *w, gpointer user_data);
+void on_matrix_spinbutton_rows_value_changed(GtkSpinButton *w, gpointer user_data);
+
+void on_matrix_radiobutton_matrix_toggled(GtkToggleButton *w, gpointer user_data);
+void on_matrix_radiobutton_vector_toggled(GtkToggleButton *w, gpointer user_data);
+
 void on_csv_import_radiobutton_matrix_toggled(GtkToggleButton *w, gpointer user_data);
 void on_csv_import_radiobutton_vectors_toggled(GtkToggleButton *w, gpointer user_data);
 void on_csv_import_optionmenu_delimiter_changed(GtkOptionMenu *w, gpointer user_data);
