@@ -1,7 +1,7 @@
 /*
     Qalculate
 
-    Copyright (C) 2003  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2006  Niklas Knutsson (nq@altern.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@
 
 #include <deque>
 
-extern GladeXML *main_glade, *about_glade, *argumentrules_glade, *csvimport_glade, *csvexport_glade, *datasetedit_glade, *datasets_glade, *nbexpression_glade, *decimals_glade;
-extern GladeXML *functionedit_glade, *functions_glade, *matrixedit_glade, *namesedit_glade, *nbases_glade, *plot_glade, *precision_glade;
+extern GladeXML *main_glade, *about_glade, *argumentrules_glade, *csvimport_glade, *csvexport_glade, *datasetedit_glade, *datasets_glade, *setbase_glade, *decimals_glade;
+extern GladeXML *functionedit_glade, *functions_glade, *matrixedit_glade, *matrix_glade, *namesedit_glade, *nbases_glade, *plot_glade, *precision_glade;
 extern GladeXML *preferences_glade, *unit_glade, *unitedit_glade, *units_glade, *unknownedit_glade, *variableedit_glade, *variables_glade;
 extern GladeXML *periodictable_glade;
 extern vector<mode_struct> modes;
@@ -68,7 +68,7 @@ GtkListStore *tDataProperties_store;
 GtkWidget *tNames;
 GtkListStore *tNames_store;
 
-GtkWidget *tabs, *expander_keypad, *expander_history;
+GtkWidget *tabs, *expander_keypad, *expander_history, *expander_stack;
 GtkEntryCompletion *completion;
 GtkListStore *completion_store;
 
@@ -80,18 +80,24 @@ GtkListStore *tSubfunctions_store;
 GtkWidget *tPlotFunctions;
 GtkListStore *tPlotFunctions_store;
 
-GtkCellRenderer *renderer;
-GtkTreeViewColumn *column;
+GtkWidget *tMatrixEdit, *tMatrix;
+GtkListStore *tMatrixEdit_store, *tMatrix_store;
+extern vector<GtkTreeViewColumn*> matrix_edit_columns, matrix_columns;
+
+GtkCellRenderer *renderer, *register_renderer;
+GtkTreeViewColumn *column, *register_column;
 GtkTreeSelection *selection;
 
 GtkWidget *expression;
 GtkWidget *resultview;
 GtkWidget *historyview;
+GtkWidget *stackview;
+GtkListStore *stackstore;
 GtkWidget *statuslabel_l, *statuslabel_r;
 GtkWidget *f_menu ,*v_menu, *u_menu, *u_menu2, *recent_menu;
 GtkAccelGroup *accel_group;
 
-extern bool show_buttons, show_history;
+extern bool show_keypad, show_history, show_stack;
 extern bool save_mode_on_exit, save_defs_on_exit, load_global_defs, hyp_is_on, inv_is_on, fetch_exchange_rates_at_startup;
 extern bool display_expression_status, enable_completion;
 extern bool use_custom_result_font, use_custom_expression_font, use_custom_status_font;
@@ -101,9 +107,9 @@ extern string status_error_color, status_warning_color;
 extern PrintOptions printops;
 extern EvaluationOptions evalops;
 
-extern vector<vector<GtkWidget*> > element_entries;
+extern bool rpn_mode, rpn_keypad_only;
 
-GtkTooltips *periodic_tooltips;
+GtkTooltips *periodic_tooltips, *main_tooltips;
 
 extern GdkPixbuf *icon_pixbuf;
 
@@ -139,7 +145,9 @@ void set_assumptions_items(AssumptionType at, AssumptionSign as) {
 	}
 }
 
-void set_mode_items(const PrintOptions &po, const EvaluationOptions &eo, AssumptionType at, AssumptionSign as, int precision, bool initial_update) {	
+void set_mode_items(const PrintOptions &po, const EvaluationOptions &eo, AssumptionType at, AssumptionSign as, bool in_rpn_mode, int precision, bool initial_update) {
+
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_rpn_mode")), in_rpn_mode);
 
 	switch(eo.approximation) {
 		case APPROXIMATION_EXACT: {
@@ -198,7 +206,7 @@ void set_mode_items(const PrintOptions &po, const EvaluationOptions &eo, Assumpt
 	}
 
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_read_precision")), eo.parse_options.read_precision != DONT_READ_PRECISION);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_rpn_mode")), eo.parse_options.rpn);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_rpn_syntax")), eo.parse_options.rpn);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_limit_implicit_multiplication")), eo.parse_options.limit_implicit_multiplication);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_assume_nonzero_denominators")), eo.assume_denominators_nonzero);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_warn_about_denominators_assumed_nonzero")), eo.warn_about_denominators_assumed_nonzero);
@@ -218,28 +226,25 @@ void set_mode_items(const PrintOptions &po, const EvaluationOptions &eo, Assumpt
 		}
 	}
 
-	if(po.base >= 2 && po.base <= 36) {
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget (main_glade, "number_base_spinbutton_base")), po.base);
-	}
 	switch(po.base) {
 		case BASE_BINARY: {
 			if(initial_update) gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget (main_glade, "combobox_base")), 0);
-			if(initial_update) gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_binary")), TRUE);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_binary")), TRUE);
 			break;
 		}
 		case BASE_OCTAL: {
 			if(initial_update) gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget (main_glade, "combobox_base")), 1);
-			if(initial_update) gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_octal")), TRUE);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_octal")), TRUE);
 			break;
 		}
 		case BASE_DECIMAL: {
 			if(initial_update) gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget (main_glade, "combobox_base")), 2);
-			if(initial_update) gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_decimal")), TRUE);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_decimal")), TRUE);
 			break;
 		}
 		case BASE_HEXADECIMAL: {
 			if(initial_update) gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget (main_glade, "combobox_base")), 3);
-			if(initial_update) gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_hexadecimal")), TRUE);
+			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_hexadecimal")), TRUE);
 			break;
 		}
 		case BASE_ROMAN_NUMERALS: {
@@ -259,8 +264,9 @@ void set_mode_items(const PrintOptions &po, const EvaluationOptions &eo, Assumpt
 		}
 		default: {
 			if(initial_update) gtk_combo_box_set_active(GTK_COMBO_BOX(glade_xml_get_widget (main_glade, "combobox_base")), 7);
-			if(initial_update) gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_custom_base")), TRUE);
-			
+			if(initial_update) gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget (main_glade, "menu_item_custom_base")), TRUE);			
+			printops.base = po.base;
+			output_base_updated_from_menu();
 		}
 	}
 	
@@ -347,31 +353,31 @@ void set_mode_items(const PrintOptions &po, const EvaluationOptions &eo, Assumpt
 		printops.short_multiplication = po.short_multiplication;
 		printops.excessive_parenthesis = po.excessive_parenthesis;
 		evalops.calculate_functions = eo.calculate_functions;
-		if(nbexpression_glade) {
+		if(setbase_glade) {
 			switch(eo.parse_options.base) {
 				case BASE_BINARY: {
-					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_binary")), TRUE);
+					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_binary")), TRUE);
 					break;
 				}
 				case BASE_OCTAL: {
-					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_octal")), TRUE);
+					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_octal")), TRUE);
 					break;
 				}
 				case BASE_DECIMAL: {
-					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_decimal")), TRUE);
+					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_decimal")), TRUE);
 					break;
 				}
 				case BASE_HEXADECIMAL: {
-					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_hexadecimal")), TRUE);
+					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_hexadecimal")), TRUE);
 					break;
 				}
 				case BASE_ROMAN_NUMERALS: {
-					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_roman")), TRUE);
+					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_roman")), TRUE);
 					break;
 				}
 				default: {
-					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_custom_base")), TRUE);
-					gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_spinbutton_custom_base")), eo.parse_options.base);
+					gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_other")), TRUE);
+					gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_input_other")), eo.parse_options.base);
 				}
 			}
 		} else {
@@ -381,9 +387,7 @@ void set_mode_items(const PrintOptions &po, const EvaluationOptions &eo, Assumpt
 
 }
 
-void
-create_main_window (void)
-{
+void create_main_window (void) {
 	
 	gchar *gstr = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, "glade", "main.glade", NULL);
 	main_glade = glade_xml_new(gstr, NULL, NULL);
@@ -401,6 +405,7 @@ create_main_window (void)
 	expression = glade_xml_get_widget (main_glade, "expression");
 	resultview = glade_xml_get_widget (main_glade, "resultview");
 	historyview = glade_xml_get_widget (main_glade, "history");
+	stackview = glade_xml_get_widget (main_glade, "stackview");
 	statuslabel_l = glade_xml_get_widget (main_glade, "label_status_left");
 	statuslabel_r = glade_xml_get_widget (main_glade, "label_status_right");
 	gtk_text_buffer_create_tag(gtk_text_view_get_buffer(GTK_TEXT_VIEW(glade_xml_get_widget (main_glade, "history"))), "history_parse", "foreground", "gray40", "style", PANGO_STYLE_ITALIC, NULL);
@@ -418,7 +423,7 @@ create_main_window (void)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (main_glade, "button_hyp")), hyp_is_on);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget (main_glade, "button_inv")), inv_is_on);
 	
-	set_mode_items(printops, evalops, CALCULATOR->defaultAssumptions()->type(), CALCULATOR->defaultAssumptions()->sign(), CALCULATOR->getPrecision(), true);
+	set_mode_items(printops, evalops, CALCULATOR->defaultAssumptions()->type(), CALCULATOR->defaultAssumptions()->sign(), rpn_mode, CALCULATOR->getPrecision(), true);
 
 	set_unicode_buttons();
 
@@ -457,15 +462,19 @@ create_main_window (void)
 
 	expander_keypad = glade_xml_get_widget(main_glade, "expander_keypad");
 	expander_history = glade_xml_get_widget(main_glade, "expander_history");
-	gtk_expander_set_expanded(GTK_EXPANDER(expander_keypad), show_buttons);
-	gtk_expander_set_expanded(GTK_EXPANDER(expander_history), show_history && !show_buttons);
+	expander_stack = glade_xml_get_widget(main_glade, "expander_stack");
 	tabs = glade_xml_get_widget(main_glade, "tabs");
-	if(show_history) {
-		gtk_notebook_set_current_page(GTK_NOTEBOOK(tabs), 1);
-	} else {
+	if(show_stack && rpn_mode) {
+		gtk_expander_set_expanded(GTK_EXPANDER(expander_stack), TRUE);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(tabs), 2);
+		gtk_widget_show(tabs);
+	} else if(show_keypad) {
+		gtk_expander_set_expanded(GTK_EXPANDER(expander_keypad), TRUE);
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(tabs), 0);
-	}
-	if(show_history || show_buttons) {
+		gtk_widget_show(tabs);
+	} else if(show_history) {
+		gtk_expander_set_expanded(GTK_EXPANDER(expander_history), TRUE);
+		gtk_notebook_set_current_page(GTK_NOTEBOOK(tabs), 1);
 		gtk_widget_show(tabs);
 	} else {
 		gtk_widget_hide(tabs);
@@ -545,6 +554,35 @@ create_main_window (void)
 		
 	}
 
+	stackstore = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	gtk_tree_view_set_model(GTK_TREE_VIEW(stackview), GTK_TREE_MODEL(stackstore));
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(stackview));
+	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
+	renderer = gtk_cell_renderer_text_new();
+	g_object_set (G_OBJECT(renderer), "xalign", 0.5, NULL);
+	column = gtk_tree_view_column_new_with_attributes(_("Index"), renderer, "text", 0, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(stackview), column);
+	register_renderer = gtk_cell_renderer_text_new();
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 6
+	g_object_set (G_OBJECT(register_renderer), "editable", TRUE, "mode", GTK_CELL_RENDERER_MODE_EDITABLE, NULL);
+#else
+	g_object_set (G_OBJECT(register_renderer), "editable", TRUE, "ellipsize", PANGO_ELLIPSIZE_END, "xalign", 1.0, "mode", GTK_CELL_RENDERER_MODE_EDITABLE, NULL);
+#endif
+	g_signal_connect((gpointer) register_renderer, "edited", G_CALLBACK(on_stackview_item_edited), NULL);
+	register_column = gtk_tree_view_column_new_with_attributes(_("Value"), register_renderer, "text", 1, NULL);
+	gtk_tree_view_append_column(GTK_TREE_VIEW(stackview), register_column);
+	g_signal_connect((gpointer) selection, "changed", G_CALLBACK(on_stackview_selection_changed), NULL);
+
+	main_tooltips = gtk_tooltips_new();
+
+	if(rpn_mode) {
+		gtk_button_set_label(GTK_BUTTON(glade_xml_get_widget(main_glade, "button_equals")), _("Ent"));
+		gtk_tooltips_set_tip(main_tooltips, glade_xml_get_widget(main_glade, "button_equals"), _("Calculate expression and add to stack"), NULL);
+		gtk_tooltips_set_tip(main_tooltips, glade_xml_get_widget(main_glade, "button_execute"), _("Calculate expression and add to stack"), NULL);
+	} else {
+		gtk_widget_hide(expander_stack);
+	}
+
 #ifndef HAVE_LIBGNOME
 	gtk_widget_set_sensitive(glade_xml_get_widget(main_glade, "menu_item_help"), FALSE);
 #endif
@@ -585,7 +623,7 @@ create_main_window (void)
 	gtk_widget_set_sensitive(glade_xml_get_widget(main_glade, "menu_item_result_popup_meta_mode_delete"), modes.size() > 2);
 	
 	if(win_width > 0) {
-		if(show_history || show_buttons) gtk_window_resize(GTK_WINDOW(glade_xml_get_widget(main_glade, "main_window")), 1, win_height);
+		if(show_history || show_keypad || (show_stack && rpn_mode)) gtk_window_resize(GTK_WINDOW(glade_xml_get_widget(main_glade, "main_window")), 1, win_height);
 		else gtk_window_resize(GTK_WINDOW(glade_xml_get_widget(main_glade, "main_window")), win_width, win_height);
 	}
 
@@ -593,10 +631,12 @@ create_main_window (void)
 	
 	set_result_size_request();
 	
-	if(show_history || show_buttons) gtk_window_resize(GTK_WINDOW(glade_xml_get_widget(main_glade, "main_window")), 1, 1);
+	if(show_history || show_keypad || (show_stack && rpn_mode)) gtk_window_resize(GTK_WINDOW(glade_xml_get_widget(main_glade, "main_window")), 1, 1);
 	else gtk_window_resize(GTK_WINDOW(glade_xml_get_widget(main_glade, "main_window")), win_width, 1);
 
+#ifndef HAVE_LIBGNOMEUI
 	gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (main_glade, "main_window")), icon_pixbuf);
+#endif
 
 	update_status_text();
 	
@@ -650,7 +690,9 @@ get_functions_dialog (void)
 
 		update_functions_tree();
 		
+#ifndef HAVE_LIBGNOMEUI
 		gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (functions_glade, "functions_dialog")), icon_pixbuf);
+#endif
 	}
 
 	return glade_xml_get_widget (functions_glade, "functions_dialog");
@@ -705,8 +747,10 @@ get_variables_dialog (void)
 		glade_xml_signal_autoconnect(variables_glade);
 
 		update_variables_tree();
-		
+
+#ifndef HAVE_LIBGNOMEUI
 		gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (variables_glade, "variables_dialog")), icon_pixbuf);
+#endif
 
 	}
 	
@@ -774,7 +818,9 @@ get_units_dialog (void)
 		gtk_entry_set_alignment(GTK_ENTRY(glade_xml_get_widget (units_glade, "units_entry_from_val")), 1.0);
 		gtk_entry_set_alignment(GTK_ENTRY(glade_xml_get_widget (units_glade, "units_entry_to_val")), 1.0);
 		
+#ifndef HAVE_LIBGNOMEUI
 		gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (units_glade, "units_dialog")), icon_pixbuf);
+#endif
 	
 	}
 	
@@ -836,8 +882,10 @@ get_datasets_dialog (void)
 		glade_xml_signal_autoconnect(datasets_glade);
 
 		update_datasets_tree();
-		
+
+#ifndef HAVE_LIBGNOMEUI		
 		gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (datasets_glade, "datasets_dialog")), icon_pixbuf);
+#endif
 		
 	}
 
@@ -861,9 +909,13 @@ get_preferences_dialog (void)
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_save_mode")), save_mode_on_exit);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_unicode_signs")), printops.use_unicode_signs);	
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_lower_case_numbers")), printops.lower_case_numbers);	
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_lower_case_e")), printops.lower_case_e);	
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_lower_case_e")), printops.lower_case_e);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_alternative_base_prefixes")), printops.base_display == BASE_DISPLAY_ALTERNATIVE);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_spell_out_logical_operators")), printops.spell_out_logical_operators);	
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_save_defs")), save_defs_on_exit);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_rpn_keypad_only")), rpn_keypad_only);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_dot_as_separator")), evalops.parse_options.dot_as_separator);
+		if(CALCULATOR->getDecimalPoint() == ".") gtk_widget_hide(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_dot_as_separator"));
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_custom_result_font")), use_custom_result_font);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_custom_expression_font")), use_custom_expression_font);
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (preferences_glade, "preferences_checkbutton_custom_status_font")), use_custom_status_font);
@@ -1139,14 +1191,21 @@ get_matrix_edit_dialog (void)
 #ifndef HAVE_LIBGNOME
 		gtk_widget_hide(glade_xml_get_widget (matrixedit_glade, "matrix_edit_button_help"));
 #endif		
-		
-		glade_xml_signal_autoconnect(matrixedit_glade);
 
-		if(element_entries.size() == 0) {
-			element_entries.resize(1);
-			element_entries[0].push_back(glade_xml_get_widget (matrixedit_glade, "matrix_edit_entry_1x1"));
+		GType types[20000];
+		for(gint i = 0; i < 20000; i += 2) {
+			types[i] = G_TYPE_STRING;
+			types[i + 1] = GDK_TYPE_COLOR;
 		}
-	
+		tMatrixEdit_store = gtk_list_store_newv(10000, types);
+		tMatrixEdit = glade_xml_get_widget (matrixedit_glade, "matrix_edit_view");
+		gtk_tree_view_set_model (GTK_TREE_VIEW(tMatrixEdit), GTK_TREE_MODEL(tMatrixEdit_store));
+		gtk_widget_modify_base(tMatrixEdit, GTK_STATE_NORMAL, &stackview->style->bg[GTK_STATE_NORMAL]);
+		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tMatrixEdit));
+		gtk_tree_selection_set_mode(selection, GTK_SELECTION_NONE);
+
+		glade_xml_signal_autoconnect(matrixedit_glade);
+		
 	}
 
 	/* populate combo menu */
@@ -1170,6 +1229,38 @@ get_matrix_edit_dialog (void)
 
 	return glade_xml_get_widget (matrixedit_glade, "matrix_edit_dialog");
 }
+GtkWidget*
+get_matrix_dialog (void)
+{
+	if(!matrix_glade) {
+	
+		gchar *gstr = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, "glade", "matrix.glade", NULL);
+		matrix_glade = glade_xml_new(gstr, NULL, NULL);
+		g_assert(matrix_glade != NULL);
+		g_free(gstr);
+	
+		g_assert (glade_xml_get_widget (matrix_glade, "matrix_dialog") != NULL);
+
+		GType types[20000];
+		for(gint i = 0; i < 20000; i += 2) {
+			types[i] = G_TYPE_STRING;
+			types[i + 1] = GDK_TYPE_COLOR;
+		}
+		tMatrix_store = gtk_list_store_newv(10000, types);
+		tMatrix = glade_xml_get_widget (matrix_glade, "matrix_view");
+		gtk_tree_view_set_model (GTK_TREE_VIEW(tMatrix), GTK_TREE_MODEL(tMatrix_store));
+		gtk_widget_modify_base(tMatrix, GTK_STATE_NORMAL, &stackview->style->bg[GTK_STATE_NORMAL]);
+		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tMatrix));
+		gtk_tree_selection_set_mode(selection, GTK_SELECTION_NONE);
+
+		glade_xml_signal_autoconnect(matrix_glade);
+
+	}
+
+	return glade_xml_get_widget (matrix_glade, "matrix_dialog");
+	
+}
+
 GtkWidget*
 get_dataobject_edit_dialog (void)
 {
@@ -1336,54 +1427,96 @@ get_csv_export_dialog (void)
 	
 }
 
-GtkWidget* get_number_base_expression_dialog (void) {
-	if(!nbexpression_glade) {
+GtkWidget* get_set_base_dialog (void) {
+	if(!setbase_glade) {
 	
-		gchar *gstr = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, "glade", "nbexpression.glade", NULL);
-		nbexpression_glade = glade_xml_new(gstr, NULL, NULL);
-		g_assert(nbexpression_glade != NULL);
+		gchar *gstr = g_build_filename (PACKAGE_DATA_DIR, PACKAGE, "glade", "setbase.glade", NULL);
+		setbase_glade = glade_xml_new(gstr, NULL, NULL);
+		g_assert(setbase_glade != NULL);
 		g_free(gstr);
 	
-		g_assert (glade_xml_get_widget (nbexpression_glade, "number_base_expression_dialog") != NULL);
-		
+		g_assert (glade_xml_get_widget (setbase_glade, "set_base_dialog") != NULL);
+
 		switch(evalops.parse_options.base) {
 			case BASE_BINARY: {
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_binary")), TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget (nbexpression_glade, "number_base_expression_spinbutton_custom_base"), FALSE);
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_binary")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_input_other"), FALSE);
 				break;
 			}
 			case BASE_OCTAL: {
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_octal")), TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget (nbexpression_glade, "number_base_expression_spinbutton_custom_base"), FALSE);
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_octal")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_input_other"), FALSE);
 				break;
 			}
 			case BASE_DECIMAL: {
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_decimal")), TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget (nbexpression_glade, "number_base_expression_spinbutton_custom_base"), FALSE);
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_decimal")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_input_other"), FALSE);
 				break;
 			}
 			case BASE_HEXADECIMAL: {
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_hexadecimal")), TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget (nbexpression_glade, "number_base_expression_spinbutton_custom_base"), FALSE);
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_hexadecimal")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_input_other"), FALSE);
 				break;
 			}
 			case BASE_ROMAN_NUMERALS: {
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_roman")), TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget (nbexpression_glade, "number_base_expression_spinbutton_custom_base"), FALSE);
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_roman")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_input_other"), FALSE);
 				break;
 			}
 			default: {
-				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_radiobutton_custom_base")), TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget (nbexpression_glade, "number_base_expression_spinbutton_custom_base"), TRUE);
-				gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget (nbexpression_glade, "number_base_expression_spinbutton_custom_base")), evalops.parse_options.base);
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_input_other")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_input_other"), TRUE);
+				gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_input_other")), evalops.parse_options.base);
 			}
 		}
-		
-		glade_xml_signal_autoconnect(nbexpression_glade);
+		switch(printops.base) {
+			case BASE_BINARY: {
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_output_binary")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other"), FALSE);
+				break;
+			}
+			case BASE_OCTAL: {
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_output_octal")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other"), FALSE);
+				break;
+			}
+			case BASE_DECIMAL: {
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_output_decimal")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other"), FALSE);
+				break;
+			}
+			case BASE_HEXADECIMAL: {
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_output_hexadecimal")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other"), FALSE);
+				break;
+			}
+			case BASE_SEXAGESIMAL: {
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_output_sexagesimal")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other"), FALSE);
+				break;
+			}
+			case BASE_TIME: {
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_output_time")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other"), FALSE);
+				break;
+			}
+			case BASE_ROMAN_NUMERALS: {
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_output_roman")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other"), FALSE);
+				break;
+			}
+			default: {
+				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_radiobutton_output_other")), TRUE);
+				gtk_widget_set_sensitive(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other"), TRUE);
+				gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget (setbase_glade, "set_base_spinbutton_output_other")), printops.base);
+			}
+		}
+
+		glade_xml_signal_autoconnect(setbase_glade);
 	
 	}
 
-	return glade_xml_get_widget (nbexpression_glade, "number_base_expression_dialog");
+	return glade_xml_get_widget (setbase_glade, "set_base_dialog");
 }
 
 GtkWidget*
@@ -1405,7 +1538,9 @@ get_nbases_dialog (void)
 		
 		glade_xml_signal_autoconnect(nbases_glade);
 		
+#ifndef HAVE_LIBGNOMEUI
 		gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (nbases_glade, "nbases_dialog")), icon_pixbuf);
+#endif
 	
 	}
 
@@ -1448,7 +1583,9 @@ GtkWidget* get_about_dialog (void) {
 		
 		glade_xml_signal_autoconnect(about_glade);
 		
+#ifndef HAVE_LIBGNOMEUI
 		gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (about_glade, "about_dialog")), icon_pixbuf);
+#endif
 	
 	}
 
@@ -1518,7 +1655,9 @@ GtkWidget* get_plot_dialog (void) {
 
 		glade_xml_signal_autoconnect(plot_glade);
 		
+#ifndef HAVE_LIBGNOMEUI
 		gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (plot_glade, "plot_dialog")), icon_pixbuf);
+#endif
 	
 	}
 		
@@ -1610,7 +1749,7 @@ GtkWidget* get_periodic_dialog (void) {
 		periodic_tooltips = gtk_tooltips_new();
 		string tip;
 		GtkStyle *e_style[] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
-		GtkStyle *l_style;
+		GtkStyle *l_style = NULL;
 		GdkColor c_black;
 		c_black.red = 0x0000;
 		c_black.green = 0x0000;
@@ -1622,7 +1761,7 @@ GtkWidget* get_periodic_dialog (void) {
 		DataProperty *p_symbol = dc->getProperty("symbol");
 		DataProperty *p_class = dc->getProperty("class");
 		DataProperty *p_name = dc->getProperty("name");
-		int x_pos, y_pos, group;
+		int x_pos = 0, y_pos = 0, group = 0;
 		string weight;
 		for(size_t i = 1; i < 120; i++) {
 			e = dc->getObject(i2s(i));
@@ -1738,7 +1877,9 @@ GtkWidget* get_periodic_dialog (void) {
 			}
 		}
 		
+#ifndef HAVE_LIBGNOMEUI
 		gtk_window_set_icon(GTK_WINDOW(glade_xml_get_widget (periodictable_glade, "periodic_dialog")), icon_pixbuf);
+#endif
 	
 	}
 
