@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2006  Niklas Knutsson (nq@altern.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -130,13 +130,14 @@ GtkWidget* get_function_edit_dialog (void);
 GtkWidget* get_variable_edit_dialog (void);
 GtkWidget* get_unknown_edit_dialog (void);
 GtkWidget* get_matrix_edit_dialog (void);
+GtkWidget* get_matrix_dialog (void);
 GtkWidget* get_dataobject_edit_dialog (void);
 GtkWidget* get_dataset_edit_dialog (void);
 GtkWidget* get_dataproperty_edit_dialog (void);
 GtkWidget* get_names_edit_dialog (void);
 GtkWidget* get_csv_import_dialog (void);
 GtkWidget* get_csv_export_dialog (void);
-GtkWidget* get_number_base_expression_dialog (void);
+GtkWidget* get_set_base_dialog (void);
 GtkWidget* get_nbases_dialog (void);
 GtkWidget* get_about_dialog (void);
 GtkWidget* get_argument_rules_dialog (void);
