@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2003  Niklas Knutsson (nq@altern.org)
+    Copyright (C) 2003-2006  Niklas Knutsson (nq@altern.org)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,9 @@
 #ifdef HAVE_LIBGNOME
 #include <libgnome/libgnome.h>
 #endif
+#ifdef HAVE_LIBGNOMEUI
+#include <libgnomeui/libgnomeui.h>
+#endif
 #include <unistd.h>
 
 #include "support.h"
@@ -28,7 +31,7 @@
 #include "main.h"
 #include "data/icon.xpm"
 
-MathStructure *mstruct, *parsed_mstruct, *parsed_tostruct;
+MathStructure *mstruct, *matrix_mstruct, *parsed_mstruct, *parsed_tostruct;
 bool prev_result_approx;
 string *parsed_to_str;
 KnownVariable *vans[5];
@@ -55,8 +58,8 @@ extern GtkWidget *expression;
 GdkPixbuf *icon_pixbuf;
 extern PrintOptions printops;
 
-GladeXML *main_glade, *about_glade, *argumentrules_glade, *csvimport_glade, *csvexport_glade, *nbexpression_glade, *datasetedit_glade, *datasets_glade, *decimals_glade;
-GladeXML *functionedit_glade, *functions_glade, *matrixedit_glade, *namesedit_glade, *nbases_glade, *plot_glade, *precision_glade;
+GladeXML *main_glade, *about_glade, *argumentrules_glade, *csvimport_glade, *csvexport_glade, *setbase_glade, *datasetedit_glade, *datasets_glade, *decimals_glade;
+GladeXML *functionedit_glade, *functions_glade, *matrixedit_glade, *matrix_glade, *namesedit_glade, *nbases_glade, *plot_glade, *precision_glade;
 GladeXML *preferences_glade, *unit_glade, *unitedit_glade, *units_glade, *unknownedit_glade, *variableedit_glade, *variables_glade;
 GladeXML *periodictable_glade;
 
@@ -68,6 +71,13 @@ bool command_thread_started;
 bool do_timeout, check_expression_position;
 gint expression_position;
 
+#ifdef HAVE_LIBGNOME
+static poptContext pctx;
+static struct poptOption options[] = {
+	{NULL, '\0', 0, NULL, 0, NULL, NULL}
+};
+#endif
+
 int main (int argc, char **argv) {
 
 #ifdef ENABLE_NLS
@@ -76,17 +86,43 @@ int main (int argc, char **argv) {
 	textdomain (GETTEXT_PACKAGE);
 #endif
 
-#ifdef HAVE_LIBGNOME
-	gnome_program_init("qalculate-gtk", VERSION, LIBGNOME_MODULE, argc, argv, 
-					GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR,
-					NULL);
-#endif
+#ifdef HAVE_LIBGNOMEUI
+	GnomeProgram *program = gnome_program_init("qalculate-gtk", VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_POPT_TABLE, options, GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);
 
+	char *icon = gnome_program_locate_file(program, GNOME_FILE_DOMAIN_APP_PIXMAP, "qalculate.png", TRUE, NULL);
+
+	if(icon) gnome_window_icon_set_default_from_file (icon);
+	g_free (icon);
+
+	g_object_get(G_OBJECT(program), GNOME_PARAM_POPT_CONTEXT, &pctx, NULL);
+
+#else
+#ifdef HAVE_LIBGNOME
+	gnome_program_init("qalculate-gtk", VERSION, LIBGNOME_MODULE, argc, argv, GNOME_PARAM_POPT_TABLE, options, GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);
+	g_object_get(G_OBJECT(program), GNOME_PARAM_POPT_CONTEXT, &pctx, NULL);
+#else
 	gtk_init(&argc, &argv);
+#endif
+#endif
 
 	glade_init();
 
 	string calc_arg;
+#ifdef HAVE_LIBGNOME
+	const char **args = poptGetArgs (pctx);
+	for(int i = 0; args && args[i]; i++) {
+		if(i > 1) {
+			calc_arg += " ";
+		}
+		if(strlen(args[i]) >= 2 && ((args[i][0] == '\"' && args[i][strlen(args[i]) - 1] == '\"') || (args[i][0] == '\'' && args[i][strlen(args[i]) - 1] == '\''))) {
+			calc_arg += args[i] + 1;
+			calc_arg.erase(calc_arg.length() - 1);
+		} else {
+			calc_arg += args[i];
+		}
+	}
+	poptFreeContext (pctx);
+#else	
 	for(int i = 1; i < argc; i++) {
 		if(i > 1) {
 			calc_arg += " ";
@@ -98,16 +134,15 @@ int main (int argc, char **argv) {
 			calc_arg += argv[i];
 		}
 	}
-
+#endif
 	b_busy = false;
 
-	
 	main_glade = NULL; about_glade = NULL; argumentrules_glade = NULL; 
 	csvimport_glade = NULL; datasetedit_glade = NULL; datasets_glade = NULL; decimals_glade = NULL; functionedit_glade = NULL; 
-	functions_glade = NULL; matrixedit_glade = NULL; namesedit_glade = NULL; nbases_glade = NULL; plot_glade = NULL; 
+	functions_glade = NULL; matrixedit_glade = NULL; matrix_glade = NULL; namesedit_glade = NULL; nbases_glade = NULL; plot_glade = NULL; 
 	precision_glade = NULL; preferences_glade = NULL; unit_glade = NULL; 
 	unitedit_glade = NULL; units_glade = NULL; unknownedit_glade = NULL; variableedit_glade = NULL; 
-	variables_glade = NULL;	csvexport_glade = NULL; nbexpression_glade = NULL; periodictable_glade = NULL;
+	variables_glade = NULL;	csvexport_glade = NULL; setbase_glade = NULL; periodictable_glade = NULL;
 
 	//create the almighty Calculator object
 	new Calculator();
@@ -119,6 +154,7 @@ int main (int argc, char **argv) {
 	parsed_mstruct = new MathStructure();
 	parsed_tostruct = new MathStructure();
 	parsed_tostruct->setUndefined();
+	matrix_mstruct = new MathStructure();
 	prev_result_approx = false;
 	parsed_to_str = new string;
 
@@ -206,7 +242,7 @@ int main (int argc, char **argv) {
 	
 	gtk_widget_set_sensitive(glade_xml_get_widget (main_glade, "menu_item_plot_functions"), canplot);
 	gtk_widget_set_sensitive(glade_xml_get_widget (main_glade, "menu_item_fetch_exchange_rates"), canfetch);
-		
+
 	//create dynamic menus
 	generate_units_tree_struct();
 	generate_functions_tree_struct();
